#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include "digest/digest.h"
#include "options/opt.h"

/**
 * @brief Join an array of string
 * @see https://stackoverflow.com/a/4681415
 *
 * @param foo
 * @param count
 * @return char*
 */
char *join(char *foo[], int count) {
    char *out = NULL;
    size_t total_len = 0;

    for (int i = 0; i < count; i++)
        total_len += strlen(foo[i]);
    total_len += count - 1; // seperator
    total_len++;            // string terminator

    out = malloc(total_len);
    out[0] = '\0';

    for (int i = 0; i < count; i++) {
        strcat(out, foo[i]);
        if (i < (count - 1))
            strcat(out, " ");
    }

    return out;
}

int main(int argc, char *argv[]) {
    opt *opts = verify_options(argc, argv);
    char *digest;

    switch (opts->flag) {
    case W_FLAG:
        for (int i = 0; i < opts->extra_count; i++) {
            digest = str_digest(opts->extra[i], opts->digest_method);

            printf("%s\t%s\n", digest, opts->extra[i]);
            free(digest);
        }
        break;

    case F_FLAG:
        for (int i = 0; i < opts->extra_count; i++) {
            digest = file_digest(opts->extra[i], opts->digest_method);

            printf("%s\t%s\n", digest, opts->extra[i]);
            free(digest);
        }
        break;

    case S_FLAG:
        digest = file_digest(NULL, opts->digest_method);

        printf("%s\n", digest);
        free(digest);
        break;

    default:
        char *foo = join(opts->extra, opts->extra_count);
        digest = str_digest(foo, opts->digest_method);

        printf("%s\t%s\n", digest, foo);
        free(digest);
        break;
    }

    discard_options(opts);

    exit(EXIT_SUCCESS);
}
